//require("persistentpanel");

/*var panel = require("persistentpanel").Panel({
  width: 180,
  height: 180,
  contentURL: "https://en.wikipedia.org/w/index.php?title=Jetpack&useformat=mobile"
});
 
panel.show();*/

